/* {{{ Copyright etc. */

/*  avwcpgeom.c - Copy certain parts of an AVW header

    Stephen Smith, FMRIB Image Analysis Group

    Copyright (C) 2001 University of Oxford  */

/*  CCOPYRIGHT */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss/libss.h"
#include "libss/libavw.h"

void usage(void);

/* }}} */
/* {{{ usage */

void usage(void)
{
  printf("\nUsage: avwcpgeom <source> <destination> [-d]\n\n");
  printf("-d : don't copy image dimensions\n\n");
  exit(1);
}

/* }}} */
/* {{{ main */

int main(argc,argv) 
     int argc;
     char **argv;
{
  char thestring[10000];
  struct dsr hdr, hdr2;
  FILE *fp;
  int doswap=0, i, copydim=1;

  if (argc<3)
    usage();

  if (argc>3)
    copydim=0;
    
  /* {{{ read source */

if((fp=fopen(argv[1],"rb"))==NULL)
{
  sprintf(thestring,"%s.hdr",argv[1]);
  if((fp=fopen(thestring,"rb"))==NULL)
    {
      printf("Can't find image related to %s\n", argv[1]);
      exit(1);
    }
}

fread(&hdr,1,sizeof(struct dsr),fp);
fclose(fp);

if(hdr.hk.sizeof_hdr!=348)
     swap_hdr(&hdr);

/* }}} */
  /* {{{ read destination */

sprintf(thestring,"%s",argv[2]);

if((fp=fopen(thestring,"rb"))==NULL)
{
  sprintf(thestring,"%s.hdr",argv[2]);
  if((fp=fopen(thestring,"rb"))==NULL)
    {
      printf("Can't find image related to %s\n", argv[2]);
      exit(1);
    }
}

fread(&hdr2,1,sizeof(struct dsr),fp);
fclose(fp);

if(hdr2.hk.sizeof_hdr!=348)
{
  swap_hdr(&hdr2);
  doswap=1;
}

/* }}} */
  /* {{{ copy geom */
	
memcpy(hdr2.dime.vox_units,hdr.dime.vox_units,4);
	
for(i=0;i<8;i++)
{
  hdr2.dime.pixdim[i]=hdr.dime.pixdim[i];
  if (copydim)
    hdr2.dime.dim[i]=hdr.dime.dim[i];
}

hdr2.hist.orient=hdr.hist.orient;

memcpy(hdr2.hist.originator,hdr.hist.originator,10);

/* }}} */
  /* {{{ write destination */

if ( doswap ) 
     swap_hdr(&hdr2);

if((fp=fopen(thestring,"wb"))==NULL)
{
  printf("Can't write %s\n", thestring);
  exit(1);
}

fwrite(&hdr2,1,sizeof(struct dsr),fp);
fclose(fp);

/* }}} */

  exit(0);
}

/* }}} */
