#include <math.h>
#include <iostream>
#include <string>
#include "newimage/newimageall.h"

using namespace NEWIMAGE;

void dump_float(float *fptr) {
  char *cptr;
  cptr = (char *) fptr;
  cerr << (int) *cptr << " , " << (int) *(cptr+1) << " , " 
       << (int) *(cptr+2) << " , " << (int) *(cptr+3) << " ";
}


//  void check_volume(const volume4D<float>& vol1) {
//    long int count=0;
//    float val, sum=0.0;
//    for (int t=0; t<vol1.tsize(); t++) {
//      for (int z=0; z<vol1.zsize(); z++) {
//        for (int y=0; y<vol1.ysize(); y++) {
//  	for (int x=0; x<vol1.xsize(); x++) {
//  	  count++;
//  	  val = vol1(x,y,z,t);
//  	  sum += val;
//  	}
//        }
//      }
//    }
//  }


int main(int argc, char *argv[]) 
{
 
  if (argc<2) { 
    cerr << "Usage: " << argv[0] << " <input_volume> [output_volume] [-v]" 
	 << endl; 
    return -1;
  }
  
  string outname="";
  bool verbose = false;
  if (argc>=3) {
    outname = argv[2];
    if (outname == "-v" ) {
      verbose = true;
      outname = "";
    }
  }

  string optarg;
  if (argc>=4) {
    optarg = argv[3];
    if (optarg=="-v") {
      verbose = true;
    }
  }

  volume4D<float> vol1;
  volumeinfo vinfo;
  read_volume4D(vol1,argv[1],vinfo);
  long int count=0, badcount=0, nancount=0;
  float val, sum=0;
  for (int t=0; t<vol1.tsize(); t++) {
    for (int z=0; z<vol1.zsize(); z++) {
      for (int y=0; y<vol1.ysize(); y++) {
	for (int x=0; x<vol1.xsize(); x++) {
	  count++;
	  if (verbose) {
	    cerr << "x,y,z,t = " << x <<","<<y<<","<<z<<","<<t<<" : ";
	    dump_float(&vol1(x,y,z,t));
	  }
	  char *cptr;
	  cptr = (char *) &(vol1(x,y,z,t));
	  int ival0 = (int) *cptr;
	  int ival1 = (int) *(cptr+1);
	  int ival2 = (int) *(cptr+2);
	  int ival3 = (int) *(cptr+3);

	  if ( ( (ival3==0) && ( (ival0!=0) || (ival1!=0) || (ival2!=0) ) ) ||
	       (ival3<-125)  )
	    {
	      badcount++;
	      if (verbose) {
		cerr << "BAD VALUE DETECTED - fixing it" << endl;
		dump_float((float *) cptr);  cerr << endl;
	      }
	      *(cptr+0) = 0;
	      *(cptr+1) = 0;
	      *(cptr+2) = 0;
	      *(cptr+3) = 0;
	    }

	  if (! finite(vol1(x,y,z,t))) {
	    nancount++;
	    vol1(x,y,z,t) = 0.0;
	  }

	  val = vol1(x,y,z,t);
	  sum += val;
	  if (verbose) cerr << "  ::  " << vol1(x,y,z,t) << "  ::  " << endl;
	}
	if (verbose) cerr << y << ",";
      }
      if (verbose) cerr << endl << "Z = " << z << endl;
    }
    if (verbose) cerr << endl << "T = " << t << endl;
  }
  cout << "Successfully parsed volume" << endl;
  cout << "Found " << badcount << " invalid elements out of " << count  << endl;
  cout << "Found " << nancount << " non-finite elements out of " << count  
       << endl;
  if (verbose) print_volume_info(vol1,"Volume");
  if (outname.size()>0) {
    cout << "Saving volume ... " << endl;
    save_volume4D(vol1,outname,vinfo);
    cout << "done" << endl;
  }
}
