/* {{{ Copyright etc. */

/*  avwhd.c - Show AVW header

    Stephen Smith, FMRIB Image Analysis Group  */

/* }}} */
/* {{{ defines, includes and typedefs */

#include "libss/libss.h"
#include "libss/libavw.h"

void usage(void);

/* }}} */
/* {{{ usage */

void usage(void)
{
  printf("\nUsage: avwhd <input>\n\n");
  exit(1);
}

/* }}} */
/* {{{ ShowHdr */

void ShowHdr(fileName,hdr)
     struct dsr *hdr;
     char *fileName;
{
  int i;
  char string[128];

  printf("filename       %s\n\n", fileName);

  /* Header Key */
  printf("sizeof_hdr     %d\n", hdr->hk.sizeof_hdr);
  printf("data_type      %s\n", hdr->hk.data_type);
  printf("db_name        %s\n", hdr->hk.db_name);
  printf("extents        %d\n", hdr->hk.extents);
  printf("session_error  %d\n", hdr->hk.session_error);
  printf("regular        %c\n", hdr->hk.regular);
  printf("hkey_un0       %c\n", hdr->hk.hkey_un0);

  /* Image Dimension */
  for(i=0;i<8;i++)
    printf("dim%d           %d\n", i, hdr->dime.dim[i]);
	
  strncpy(string,hdr->dime.vox_units,4);
  printf("vox_units      %s\n", string);
	
  strncpy(string,hdr->dime.cal_units,8);
  printf("cal_units      %s\n", string);
  printf("unused1        %d\n", hdr->dime.unused1);
  printf("datatype       %d\n", hdr->dime.datatype);
  printf("bitpix         %d\n", hdr->dime.bitpix);
	
  for(i=0;i<8;i++)
    printf("pixdim%d        %.10f\n",i, hdr->dime.pixdim[i]);
	
  printf("vox_offset     %6.4f\n",  hdr->dime.vox_offset);
  printf("funused1       %6.4f\n", hdr->dime.funused1);
  printf("funused2       %6.4f\n", hdr->dime.funused2);
  printf("funused3       %6.4f\n", hdr->dime.funused3);
  printf("cal_max        %6.4f\n", hdr->dime.cal_max);
  printf("cal_min        %6.4f\n", hdr->dime.cal_min);
  printf("compressed     %d\n", hdr->dime.compressed);
  printf("verified       %d\n", hdr->dime.verified);
  printf("glmax          %d\n", hdr->dime.glmax);
  printf("glmin          %d\n", hdr->dime.glmin);

  /* Data History */
  strncpy(string,hdr->hist.descrip,80);
  printf("descrip        %s\n", string);
  strncpy(string,hdr->hist.aux_file,24);
  printf("aux_file       %s\n", string);
  printf("orient         %d\n", hdr->hist.orient);

  strncpy(string,hdr->hist.originator,10);
  printf("originator     %s\n", string);
  {
    short blah[5];
    memcpy(blah,hdr->hist.originator,5*sizeof(short));
    printf("origin1        %d\n",blah[0]);
    printf("origin2        %d\n",blah[1]);
    printf("origin3        %d\n",blah[2]);
  }

  strncpy(string,hdr->hist.generated,10);
  printf("generated      %s\n", string);

  strncpy(string,hdr->hist.scannum,10);
  printf("scannum        %s\n", string);

  strncpy(string,hdr->hist.patient_id,10);
  printf("patient_id     %s\n", string);

  strncpy(string,hdr->hist.exp_date,10);
  printf("exp_date       %s\n", string);

  strncpy(string,hdr->hist.exp_time,10);
  printf("exp_time       %s\n", string);

  strncpy(string,hdr->hist.hist_un0,10);
  printf("hist_un0       %s\n", string);

  printf("views          %d\n", hdr->hist.views);
  printf("vols_added     %d\n", hdr->hist.vols_added);
  printf("start_field    %d\n", hdr->hist.start_field);
  printf("field_skip     %d\n", hdr->hist.field_skip);
  printf("omax           %d\n", hdr->hist.omax);
  printf("omin           %d\n", hdr->hist.omin);
  printf("smin           %d\n", hdr->hist.smax);
  printf("smin           %d\n", hdr->hist.smin);
}

/* }}} */
/* {{{ main */

int main(argc,argv) 
     int argc;
     char **argv;
{
  char thestring[10000], thestring2[10000];
  struct dsr hdr;
  FILE *fp;

  if (argc<2)
    usage();
    
  strcpy(thestring,argv[1]);
  if ( (strcmp(".hdr",thestring+strlen(thestring)-4)==0) ||
       (strcmp(".img",thestring+strlen(thestring)-4)==0) )
    thestring[strlen(thestring)-4]=0;

  sprintf(thestring2,"%s.hdr",thestring);
 
  if((fp=fopen(thestring2,"rb"))==NULL)
    {
      printf("Can't find image related to %s\n", argv[1]);
      exit(1);
    }

  fread(&hdr,1,sizeof(struct dsr),fp);
  fclose(fp);

  if(hdr.hk.sizeof_hdr!=348)
    {
      printf("Byte-swapping\n");
      swap_hdr(&hdr);
    }

  ShowHdr(argv[1], &hdr);

  exit(0);
}

/* }}} */
