/* {{{ copyright */

/*  avwmerge.c - concatenate AVW files into a single output

    Steve Smith, David Flitney and Stuart Clare, FMRIB Image Analysis Group

    Copyright (C) 2000-2002 University of Oxford  */

/*  CCOPYRIGHT */

/* }}} */
/* {{{ includes */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "avwio/avwio.h"

#define MAXINPUTS 10000

/* }}} */
/* {{{ usage */

void usage()
{
  printf("\nUsage: avwmerge <-x/y/z/t> <output> <file1 file2 .......>\n\n");
  printf("-t : concatenate images in time\n");
  printf("-x : concatenate images in the x direction\n");
  printf("-y : concatenate images in the y direction\n");
  printf("-z : concatenate images in the z direction\n\n");
  printf("-a : auto-choose: single slices -> volume, volumes -> 4D (time series)\n\n");
  exit(1);
}

/* }}} */

int main(int argc, char **argv)
{
  /* {{{ vars */

  AVW *src=NULL, *dest;
  short x[MAXINPUTS], y[MAXINPUTS], z[MAXINPUTS], v[MAXINPUTS],
    X=0, Y=0, Z=0, V=0,
    xx, yy, zz, vv,
    t;
  char *buffer[MAXINPUTS], *outbuffer;
  unsigned int i, direction, bpp=0;
  char filename[10000];

/* }}} */

  /* {{{ process initial args */

if (argc<4)
     usage();

if (!strcmp(argv[1], "-t"))
     direction=0;
else if (!strcmp(argv[1], "-x"))
     direction=1;
else if (!strcmp(argv[1], "-y"))
     direction=2;
else if (!strcmp(argv[1], "-z"))
     direction=3;
else if (!strcmp(argv[1], "-a"))
     direction=4;
else usage();

/* }}} */
  /* {{{ loop over all inputs */

for(i = 0; i < argc-3; i++)
{
  strcpy(filename,argv[i+3]);
  if ( (strcmp(".hdr",filename+strlen(filename)-4)==0) ||
       (strcmp(".img",filename+strlen(filename)-4)==0) )
    filename[strlen(filename)-4]=0;
  
  if((src=AvwOpen(filename,"r"))==NULL) {
    perror("avwmerge");
    exit(1);
  }
  
  AvwGetDim(src,&x[i],&y[i],&z[i],&v[i]);
  
  if (i==0)
    {
      bpp = AvwGetDataType(src, &t) / 8;
      V=v[0]; X=x[0]; Y=y[0]; Z=z[0];
      if(direction==4){
	if((z[0]<2)&&(v[0]<2))direction=3;
	else direction=0;
      }
    }
  else
    {
      if (direction==0) V+=v[i];
      if (direction==1) X+=x[i];
      if (direction==2) Y+=y[i];
      if (direction==3) Z+=z[i];
    }

  buffer[i] = malloc(x[i]*y[i]*z[i]*v[i]*bpp);
  AvwReadVolumes(src, buffer[i], v[i]);
  
  if (i<argc-4)
    AvwClose(src);
}

outbuffer = malloc(X * Y * Z * V * bpp);
xx=yy=zz=vv=0;

/*printf("%d %d %d %d\n",X,Y,Z,V);*/

for(i = 0; i < argc-3; i++)
{
  if (direction==0)
    {
      memcpy(outbuffer+X*Y*Z*vv*bpp,buffer[i],X*Y*Z*v[i]*bpp);
      vv+=v[i];
    }

  if (direction==1)
    {
      for(vv=0;vv<V;vv++)
	for(zz=0;zz<Z;zz++)
	  for(yy=0;yy<Y;yy++)
	    memcpy(outbuffer+(X*Y*Z*vv+X*Y*zz+X*yy+xx)*bpp,buffer[i]+(Y*Z*vv+Y*zz+yy)*x[i]*bpp,x[i]*bpp);
      xx+=x[i];
    }    

  if (direction==2)
    {
      for(vv=0;vv<V;vv++)
	for(zz=0;zz<Z;zz++)
	  memcpy(outbuffer+(Y*Z*vv+Y*zz+yy)*X*bpp,buffer[i]+(Z*vv+zz)*y[i]*X*bpp,X*y[i]*bpp);
      yy+=y[i];
    }    

  if (direction==3)
    {
      for(vv=0;vv<V;vv++)
	memcpy(outbuffer+(vv*Z+zz)*X*Y*bpp,buffer[i]+X*Y*z[i]*vv*bpp,X*Y*z[i]*bpp);
      zz+=z[i];
    }    
}

/* }}} */
  /* {{{ write output */

  strcpy(filename,argv[2]);
  if ( (strcmp(".hdr",filename+strlen(filename)-4)==0) ||
       (strcmp(".img",filename+strlen(filename)-4)==0) )
    filename[strlen(filename)-4]=0;

  dest = AvwOpen(filename, "w");

  AvwCloneHeader(dest, src);
  AvwClose(src);

  AvwSetDim(dest, X, Y, Z, V);
  AvwSetDimensionality(dest, 4);

  AvwWriteVolumes(dest, outbuffer, V);

  AvwClose(dest);

/* }}} */

  return(0);
}
