/*  avwswapdim.cc

    Mark Jenkinson, Howard Florey Institute, University of Melbourne

    Copyright (C) 2001-2002 Howard Florey Institute  */

/*  CCOPYRIGHT  */

#include "newimage/newimageall.h"
#include "newimage/fmribmain.h"

using namespace NEWIMAGE;

void print_usage(const string& progname) {
  cerr << "Usage: " << progname << " <input> <a> <b> <c> [output]" << endl;
  cerr << endl;
  cerr << "  where a,b,c represent the new x,y,z axes in terms of the" << endl;
  cerr << "  old axes.  They can take values of -x,x,y,-y,z,-z" << endl;
  cerr << "  e.g.  " << progname << " invol y x -z outvol" << endl;
}

int dimarg(const string& val)
{
  if (val=="x") {
    return 1;
  } else if (val=="x-" || val=="-x") {
    return -1;
  } else if (val=="y") {
    return 2;
  } else if (val=="y-" || val=="-y") {
    return -2;
  } else if (val=="z") {
    return 3;
  } else if (val=="z-" || val=="-z") {
    return -3;
  } else {
    return 0;
  }
}

template <class T>
void setrow(Matrix& affmat, int rownum, int dimnum, const volume4D<T>& invol)
{
  if (dimnum==1 || dimnum==-1) {
    affmat(rownum,1)=1*sign(dimnum); affmat(rownum,2)=0; affmat(rownum,3)=0;
  }
  if (dimnum==2 || dimnum==-2) {
    affmat(rownum,1)=0; affmat(rownum,2)=1*sign(dimnum); affmat(rownum,3)=0;
  }
  if (dimnum==3 || dimnum==-3) {
    affmat(rownum,1)=0; affmat(rownum,2)=0; affmat(rownum,3)=1*sign(dimnum);
  }
  if (dimnum>0) return;
  float fov=0.0;
  if (dimnum==-1) {
    fov = invol.xsize() * invol.xdim();
  }
  if (dimnum==-2) {
    fov = invol.ysize() * invol.ydim();
  }
  if (dimnum==-3) {
    fov = invol.zsize() * invol.zdim();
  }
  affmat(rownum,4)=fov;
}

template <class T>
Matrix getmat(const volume4D<T>& invol, int dim1, int dim2, int dim3)
{
  Matrix affmat(4,4);
  affmat = 0.0;
  affmat(4,4)=1.0;
  setrow(affmat,1,dim1,invol);
  setrow(affmat,2,dim2,invol);
  setrow(affmat,3,dim3,invol);
  return affmat;
}


template <class T>
int fmrib_main(int argc,char *argv[])
{
  string newx=argv[2], newy=argv[3], newz=argv[4];
  string inname=argv[1];
  string outname="";
  bool showmat=false;
  if (argc==6) {
    outname=argv[5];
    showmat=false;
  } else {
    showmat=true;
  }

  volumeinfo volinfo;
  volume4D<T> invol;
  read_volume4D(invol,inname,volinfo);

  Matrix affmat;
  affmat = getmat(invol,dimarg(newx),dimarg(newy),dimarg(newz));

  if (showmat) {
    cout << affmat << endl;
  }

  if (affmat.Determinant()<0.0) {
    cerr << "WARNING:: Flipping Left/Right (as det < 0)" << endl;
  }
  
  invol.swapdimensions(dimarg(newx),dimarg(newy),dimarg(newz));


  return save_volume4D(invol,outname,volinfo);
}

  
int main(int argc,char *argv[])
{
  string progname=argv[0];
  if (argc<5) { 
    print_usage(progname);
    return -1; 
  }
  
  string inname = argv[1];
  // call the templated main
  return call_fmrib_main(dtype(inname),argc,argv);
}

