#{{{ copyright and setup

#   featquery - apply masking etc to get out stats from FEAT runs
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 2002 University of Oxford
#
#   TCLCOPYRIGHT

source [ file dirname [ info script ] ]/fslstart.tcl

set VARS(history) {}

#}}}
#{{{ featquery

proc featquery { w } {
    global fmri PXHOME FSLDIR VARS argc argv PWD feat_files query vars

    toplevel $w
    wm title      $w "FEAT Stats Query"
    wm iconname   $w "FEAT Query"
    wm iconbitmap $w @${FSLDIR}/tcl/fmrib.xbm

    set fmri(multiple) 1
    set fmri(level) 1
    set fmri(analysis) 4
    set fmri(inputtype) 1

    #{{{ select FEAT directories

frame $w.select -relief raised -borderwidth 1

tixControl $w.select.number -label "Number of FEAT directories " \
	-variable fmri(multiple) -step 1 -min 1 -selectmode immediate \
	-options { entry.width 3 }

button $w.select.button -text "Select" -command "feat5:multiple_select $w -1 \"Select FEAT directories\" "

pack $w.select.number $w.select.button -in $w.select -padx 5 -pady 5 -side left -expand yes

#}}}
    #{{{ setup mask or coordinates

frame $w.mask -relief raised -borderwidth 1

FSLFileEntry $w.mask.mask \
	-variable fmri(mask) \
	-pattern "*.hdr" \
	-directory $PWD \
	-label "Mask image " \
	-title "Select the mask image" \
	-width 40 \
	-filterhist VARS(history)

frame $w.mask.type
set fmri(masktype) 1
radiobutton $w.mask.type.image -text "Use mask image"   -value 1 -variable fmri(masktype) -command "featquery_update $w"
radiobutton $w.mask.type.coord -text "Use co-ordinates" -value 2 -variable fmri(masktype) -command "featquery_update $w"
pack $w.mask.type.image $w.mask.type.coord -in $w.mask.type -padx 5 -side left -expand yes

pack $w.mask.mask $w.mask.type -in $w.mask -padx 5 -pady 5 -side top


frame $w.mask.coords

set fmri(coordtype) -vox
radiobutton $w.mask.coords.vox -text "vox" -value -vox -variable fmri(coordtype)
radiobutton $w.mask.coords.mm  -text "mm"  -value -mm  -variable fmri(coordtype)

set fmri(cX) 0
set fmri(cY) 0
set fmri(cZ) 0
tixControl $w.mask.coords.cX -label "  X " -variable fmri(cX) -step 1 -selectmode immediate -options { entry.width 5 }
tixControl $w.mask.coords.cY -label "Y " -variable fmri(cY) -step 1 -selectmode immediate -options { entry.width 5 }
tixControl $w.mask.coords.cZ -label "Z " -variable fmri(cZ) -step 1 -selectmode immediate -options { entry.width 5 }
pack $w.mask.coords.vox $w.mask.coords.mm $w.mask.coords.cX $w.mask.coords.cY $w.mask.coords.cZ -in $w.mask.coords -padx 5 -side left -expand yes

#}}}
    #{{{ options

frame $w.opts -relief raised -borderwidth 1

#{{{ thresholding

frame $w.opts.thresh

set fmri(statsthresh_yn) 0
checkbutton $w.opts.thresh.yn -variable fmri(statsthresh_yn) -text "Apply threshold" -command "featquery_update $w"

set fmri(statsthresh) 0
tixControl $w.opts.thresh.thresh -variable fmri(statsthresh) -step 1 -selectmode immediate -options { entry.width 5 }

pack $w.opts.thresh.yn $w.opts.thresh.thresh  -in $w.opts.thresh -padx 5 -side left

#}}}
#{{{ interp thresholding

frame $w.opts.ithresh

set fmri(ithresh_yn) 0
checkbutton $w.opts.ithresh.yn -variable fmri(ithresh_yn) -text "Change mask post-interpolation thresholding" -command "featquery_update $w"

set fmri(ithresh) 0.5
tixControl $w.opts.ithresh.thresh -variable fmri(ithresh) -step 0.1 -min 0 -max 1 -selectmode immediate -options { entry.width 5 }

pack $w.opts.ithresh.yn $w.opts.ithresh.thresh  -in $w.opts.ithresh -padx 5 -side left

#}}}

pack $w.opts.thresh $w.opts.ithresh -in $w.opts -side top

#}}}
    #{{{ bottom buttons

frame $w.btns -relief raised -borderwidth 1

button $w.btns.apply  -text "Go"   -command "featquery_proc $w"
button $w.btns.cancel -text "Exit" -command "destroy $w"
button $w.btns.help   -text "Help" -command "FmribWebHelp file: ${FSLDIR}/doc/feat5/featquery.html"

pack $w.btns.apply $w.btns.cancel $w.btns.help -in $w.btns -padx 5 -pady 5 -side left -expand yes

#}}}

    pack $w.select $w.mask $w.opts $w.btns -in $w -padx 5 -pady 5 -fill x 
    featquery_update $w
}

#}}}
#{{{ featquery_update

proc featquery_update { w } {
    global fmri

    pack forget $w.mask.coords
    if { $fmri(masktype) == 2 } {
	pack $w.mask.coords -in $w.mask -padx 5 -pady 5 -side top
    }

    pack forget $w.opts.thresh.thresh
    if { $fmri(statsthresh_yn) } {
	pack $w.opts.thresh.thresh -in $w.opts.thresh -after $w.opts.thresh.yn -padx 5 -side left
    }

    pack forget $w.opts.ithresh.thresh
    if { $fmri(ithresh_yn) } {
	pack $w.opts.ithresh.thresh -in $w.opts.ithresh -after $w.opts.ithresh.yn -padx 5 -side left
    }

}

#}}}
#{{{ featquery_whichstats

proc featquery_whichstats { w } {
    global fmri feat_files PWD

    if { [ winfo exists $w.stats ] } {
	destroy $w.stats
    }
    frame $w.stats -relief raised -borderwidth 1
    pack $w.stats -in $w -after $w.select -padx 5 -pady 5 -fill x

    #{{{ setup scrollbar viewport as $v

    set w0 $w.stats

    frame $w0.f
    pack $w0.f -expand yes -fill both -in $w0 -side top
    canvas $w0.f.viewport -yscrollcommand "$w0.f.ysbar set" -xscrollcommand "$w0.xsbar set"
    scrollbar $w0.xsbar -command "$w0.f.viewport xview" -orient horizontal
    scrollbar $w0.f.ysbar -command "$w0.f.viewport yview" -orient vertical
    frame $w0.f.viewport.f
    $w0.f.viewport create window 0 0 -anchor nw -window $w0.f.viewport.f
    bind $w0.f.viewport.f <Configure> "feat5:scrollform_resize $w0 $w0.f.viewport"
    pack $w0.f.viewport -side left -fill both -expand true -in $w0.f

    set v $w0.f.viewport.f

#}}}

    cd $feat_files(1)

    set row 0

    set fmri(nstats) 0
    set fmri(statslist) ""

    foreach statstype { filtered_func_data stats/pe stats/cope stats/tstat stats/fstat stats/zstat stats/zfstat thresh_zstat thresh_zfstat } {
    
	set statslist [ glob -nocomplain ${statstype}.hdr ${statstype}?.hdr ${statstype}??.hdr ${statstype}???.hdr ]
    
	if { [ llength $statslist ] > 0 } {

	    label $v.${row}_0 -text "$statstype  "
	    grid $v.${row}_0 -in $v -column 0 -row $row
	    
	    set column 1

	    regsub -all ".hdr" $statslist "" statslist

	    foreach stats $statslist {
		set fmri(dostats.$fmri(nstats)) 0
		set i [ string trimleft $stats "abcdefghijklmnopqrstuvwxyz_/" ]
		checkbutton $v.${row}_$column -variable fmri(dostats.$fmri(nstats)) -text "$i "
		grid $v.${row}_$column -in $v -column $column -row $row
		incr column 1

		set fmri(statslist) "$fmri(statslist) $stats"
		incr fmri(nstats) 1
	    }
	    
	    incr row 1
	}
	
    }

    checkbutton $w.stats.percent -variable fmri(showpercent) -text "Convert PE/COPE values to % (must rescale if not block-design)"
    pack $w.stats.percent -in $w.stats -padx 5 -pady 5 -side top -expand yes

    cd $PWD
}

#}}}
#{{{ featquery_proc

proc featquery_proc { w } {
    global fmri feat_files FSLDIR

    if { ( $fmri(mask) == "" ) ||
	 ( ! [ file exists $fmri(mask) ] && ! [ file exists $fmri(mask).hdr ] &&
	   ! [ file exists $feat_files(1)/$fmri(mask) ] && ! [ file exists $feat_files(1)/$fmri(mask).hdr ] ) } {
	puts "\n\nYou need to set the mask image!\n\n"
	return
    }

    set thecommand "${FSLDIR}/bin/featquery $fmri(multiple)"

    for { set n 1 } { $n <= $fmri(multiple) } { incr n 1 } {
	set thecommand "$thecommand $feat_files($n)"
    }

    set nstats 0
    set statslist ""
    for { set n 0 } { $n < $fmri(nstats) } { incr n 1 } {
	if { $fmri(dostats.$n) == 1 } {
	    set statslist "$statslist [ lindex $fmri(statslist) $n ]"
	    incr nstats 1
	}
    }
    set thecommand "$thecommand $nstats $statslist"

    if { $fmri(showpercent) == 1 } {
	set thecommand "$thecommand -p"
    }

    if { $fmri(statsthresh_yn) } {
	set thecommand "$thecommand -t $fmri(statsthresh)"
    }

    if { $fmri(ithresh_yn) } {
	set thecommand "$thecommand -i $fmri(ithresh)"
    }

    set thecommand "$thecommand $fmri(mask)"

    if { $fmri(masktype) == 2 } {
	set thecommand "$thecommand $fmri(coordtype) $fmri(cX) $fmri(cY) $fmri(cZ)"
    }

    fsl:exec $thecommand
}

#}}}
#{{{ call GUI and wait

if { ! [ info exists INGUI ] } {
    wm withdraw .
    featquery .r
    tkwait window .r
}

#}}}
