/*  FilmGlsOptions.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  CCOPYRIGHT  */

#if !defined(__FilmGlsOptions_h)
#define __FilmGlsOptions_h

#include <string>
#include <math.h>
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include <stdio.h>
#include "utils/log.h"

using namespace Utilities;

namespace FILM {

class FilmGlsOptions {
 public:
  static FilmGlsOptions& getInstance();
  ~FilmGlsOptions() { delete gopt; }

  string inputfname;
  string paradigmfname;
  string epifname;
  string datadir;
  string susanpath;

  string neffsfname;
  
  bool verbose;
  bool smoothACEst;
  bool fitAutoRegressiveModel;
  bool tukey;
  bool pava;
  bool multitaper;
  bool noest;
  bool ac_only;

  int tukeysize;
  float multitapersize;

  int trimscans;
  float thresh;
  int ms;
  short maxshort;
  int epith;

  void parse_command_line(int argc, char** argv, Log& logger);

 private:
  FilmGlsOptions();
  
  const FilmGlsOptions& operator=(FilmGlsOptions&);
  FilmGlsOptions(FilmGlsOptions&);
      
  static FilmGlsOptions* gopt;

  void print_usage(int argc, char *argv[]);
  
};

inline FilmGlsOptions& FilmGlsOptions::getInstance(){
  if(gopt == NULL)
    gopt = new FilmGlsOptions();
  
  return *gopt;
}

inline FilmGlsOptions::FilmGlsOptions()
{ 
  // set up defaults
  neffsfname = "neffs";
  datadir = "results";
  susanpath = "/usr/local/fsl/bin/susan_smooth";
  epifname = "epivolume";
  thresh = 0;
  inputfname = "";  
  paradigmfname = "";
  epith = 0;
  
  ms = 4;
  maxshort = 32000;
  fitAutoRegressiveModel = false;
  tukeysize = 0;  
  multitapersize = 4.0;
  
  smoothACEst = false;
  verbose = false;
  pava = false;
  tukey = true;
  multitaper = false;
  noest = false;  
  ac_only = false;
}

}

#endif










