/*  glimGls.h

    Mark Woolrich, FMRIB Image Analysis Group

    Copyright (C) 1999-2000 University of Oxford  */

/*  CCOPYRIGHT  */

#if !defined(__glimgls_h)
#define __glimgls_h

#include <iostream>
#include <fstream>
#define WANT_STREAM
#define WANT_MATH

#include "newmatap.h"
#include "newmatio.h"
#include "Volume.h"
#include "VolumeSeries.h"

using namespace NEWMAT;

namespace FILM {

#define BATCHSIZE 50

  class GlimGls
    {
    public:
      GlimGls(const int pnumTS, const int psizeTS, const int pnumParams) : 
	numTS(pnumTS),
	sizeTS(psizeTS),
	numParams(pnumParams),
	corrections(numParams*numParams,numTS),
	b(numParams, numTS),
	sigmaSquareds(numTS),
	dof(sizeTS - numParams)
	{}
     
      void setData(const ColumnVector& p_y, const Matrix& p_x, const int ind);       
      void Save(const VolumeSeries::Dims& pdims, const ColumnVector& prethreshpos);
      ColumnVector& getResiduals() { return r; }
      void CleanUp();

    private:

      void SetCorrection(const Matrix& corr, const int ind);

      GlimGls(const GlimGls&);
      GlimGls& operator=(const GlimGls& p_glimgls);
   
      int numTS;
      int sizeTS;
      int numParams;

      // Data to be saved:
      VolumeSeries corrections;
      Matrix b;
      Volume sigmaSquareds;
      float dof;
      ColumnVector r;
    };

}

#endif




