/*  unwarpfns.h

    Mark Jenkinson, FMRIB Image Analysis Group

    Copyright (C) 2000 University of Oxford  */

/*  CCOPYRIGHT  */

#if !defined(__unwarpfns_h)
#define __unwarpfns_h

#include <string>
#include <iostream>
#include <fstream>
#define WANT_STREAM
#define WANT_MATH

#include <vector>
#include <map>
#include <algorithm>
#include "newmatap.h"
#include "newmatio.h"
#include "newimage/newimageall.h"

using namespace NEWIMAGE;
using namespace NEWMAT;

//////////////////////////////////////////////////////////////////////////

int maximum_position(const ColumnVector& vec);

int maximum_position(const Matrix& vec);

void delete_row(Matrix& mat, int r);

void delete_column(Matrix& mat, int c);

void insert_row(Matrix& mat, int r, const Matrix& nrow);

void insert_column(Matrix& mat, int c, const Matrix& ncol);

ColumnVector pava(const ColumnVector& data, const ColumnVector& weights);

ColumnVector pava(const ColumnVector& data);

//////////////////////////////////////////////////////////////////////////

template <class T>
void simple_erode(volume<T>& binaryvol);
  
template <class T>
void simple_dilate(volume<T>& binaryvol);

volume<float> despike_filter2D(const volume<float>& vol, float threshold);

volume<float> median_filter2D(const volume<float>& vol);

volume<float> masked_despike_filter2D(const volume<float>& vol, 
				      const volume<float>& mask,
				      float threshold);

volume<float> masked_median_filter2D(const volume<float>& vol, 
				     const volume<float>& mask);

float basic_mask_threshold(const volume<float>& absmap);

volume<float> make_basic_head_mask(const volume<float>& absmap, float thresh);

volume<float> make_head_mask(const volume<float>& absmap, float thresh); 
volume<float> make_head_mask2D(const volume<float>& absmap, float thresh);

volume<float> make_filled_head_mask(const volume<float>& absmap); 

volume<float> fill_head_mask(const volume<float>& origmask);

void fill_holes(volume<float>& vol, const volume<float>& holemask, 
		const volume<float>& mask); 

volume<float> extrapolate_volume(const volume<float>& datavol,
				 const volume<float>& origmask,
				 const volume<float>& enlargedmask);

volume<float> polynomial_extrapolate(const volume<float>& datavol,
				     const volume<float>& validmask,
				     int n, bool verbose=false);

volume<float> fourier_extrapolate(const volume<float>& datavol,
				  const volume<float>& validmask,
				  int n, bool verbose=false);

void connection_matrices(const volume<float>& phasemap,
			 const volume<int>& label, 
			 Matrix& Qab, Matrix& Pab, Matrix& Nab, double& K);

void connection_matrices(const volume<float>& phasemap,
			 const volume<int>& label, Matrix& Kab);

float calc_cost(const Matrix& Mab, const Matrix& Qab, const Matrix& Pvec, 
		float K); 

float calc_delta_cost(const Matrix& Mab, const Matrix& Qab, const Matrix& Pvec,
		      int j, float d); 

float calc_delta_cost(const Matrix& Mab, const Matrix& Qab, 
		      const Matrix& Pvecon2PI, int signedj); 

////////////////////////////////////////////////////////////////////////////

volume<float> apply_unwrapping(const volume<float>& phasemap, 
			       const volume<int>& label, 
			       const Matrix& label_offsets);

volume<int> find_phase_labels(const volume<float>& phasemap, 
			      const volume<float>& mask,
			      int n=4);

volume<int> find_phase_labels2D(const volume<float>& phasemap, 
				const volume<float>& mask,
				int n, bool unique3Dlabels); 
 
volume<float> unwrap(const volume<float>& phasemap, 
		     const volume<int>& label, bool verbose=false);

volume<float> unwrap2D(const volume<float>& phasemap, 
		       const volume<int>& label, bool verbose=false);

volume<float> calc_fmap(const volume<float>& phase1, 
			const volume<float>& phase2,
			const volume<float>& mask1,
			const volume<float>& mask2,
			float asym_time);

float fmap2pixshift_factor(const volume<float>& fmap, float pe_dwell_time,
			   const string& dir);

volume<float> yderiv(const volume<float>& vol);

volume<float> limit_pixshift(const volume<float>& pixshift, 
			     const volume<float>& mask, float mindiff);

volume<float> apply_pixshift(const volume<float>& epivol,
			     const volume<float>& pixshiftmap);

////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////


// TEMPLATE BODIES

template <class T>
void simple_erode(volume<T>& binaryvol)
{
  binaryvol.setpadvalue(1.0);
  binaryvol.setextrapolationmethod(constpad);
  ColumnVector structelement(3);
  structelement = 1.0/3.0;
  binaryvol = convolve_separable(binaryvol,structelement,
				 structelement,structelement);
  binaryvol.binarise(1.0 - 0.5/27.0);
}
  
template <class T>
void simple_dilate(volume<T>& binaryvol)
{
  binaryvol.setpadvalue(0.0);
  binaryvol.setextrapolationmethod(constpad);
  ColumnVector structelement(3);
  structelement = 1.0/3.0;
  binaryvol = convolve_separable(binaryvol,structelement,
				 structelement,structelement);
  binaryvol.binarise(0.5/27.0,100.0);
}

#endif
