#

# clusterrep 

# Copyright (C) 1999 John O'Doherty
# Dept. of Exp. Psy., Oxford. E-mail:john.odoherty@psy.ox.ac.uk

# TCLCOPYRIGHT

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

set outname "/tmp/${USER}cluster.txt"

MxSetTextDefaults {{OutlineColor none} {FillColor none} {TextColor green} {HorzJustify Center} {VertJustify Bottom} {NameFont -adobe-courier-bold-r-normal--8-80-75-75-m-50-iso8859-1}}

#Ask user to input 3 page names: (1) thresholded z-score (2) % change map name
#(3) fused reference map (4) cluster output name

InputFromWidget "Cluster output" 5 "Thresholded z-score image" zimage 20 "%change activation image" actimg 20 "Fused reference image" refimg 20 "Image Mask" ImageMask  20 "Cluster output file" outname 20

set annotate [YesNoWidget "Annotate Fused Image with Cluster numbers?" Yes No]


MxGetCurrentFolder Folder
MxGetPageByName $Folder $ImageMask Mask

#First get z-score image and threshold it
MxGetPageByName $Folder $zimage zpage

# Make sure that no blobs touch the edges - MEDx cluster detection
# doen't work if they do. Then make real
MxGetImageProperties $zpage pl
set bpp [keylget pl Bits]
set ww [keylget pl Width]
set hh [keylget pl Height]
set ss [keylget pl Slices]
set wwh [expr ( $ww - 1 )/2]
set hhh [expr ( $hh - 1 )/2]
set ssh [expr ( $ss - 1 )/2]
incr ww 10
incr hh 10
incr ss 4
keylset newlist Cx $wwh Cy $hhh Cz $ssh Rx 0 Ry 0 Rz 0 Sx 1 Sy 1 Sz 1 Tx 0 Ty 0 Tz 0 Dx $ww Dy $hh Dz $ss RotMethod M
MxInterpolationMode NN
MxReslice $zpage newlist zpagedup
MxGetPageByName $Folder $actimg actpag
MxReslice $actpag newlist actpagedup
MxGetPageByName $Folder $refimg refpage
MxSaveLUT $refpage "/tmp/${USER}templut"
MxReslice $refpage newlist refpagedup
MxReslice $Mask newlist maskdup

MxThreshold $zpagedup 1 B1

#set to 2DALL because of bug in Digital v. of MEDX
MxSetImageOpMode $actpagedup 2DAll

MxExpr "{padding ZP} {InOut In} {Bounds C} {gain 1} {KSize 3} {low 0} {high 1} {AllSlices true}" $actpagedup = "($actpagedup * $zpagedup)"
MxSetImageOpMode $actpagedup 3D
MxDeletePage $zpagedup

# Steve Smiths's trick from cluster_count_procs.tcl!
# Fool medx into thinking that this is a z score image so it doesn't 
# prompt with the "did you know this isn't a z score?" question
#MxDupPage $actpagedup actpagedup2
#MxGroupImages { $actpagedup $actpagedup2 } I "New Group1" NG1
#MxBetweenGrpStatistics $NG1 $NG1 U I0 I2 I3 I4 I5
#MxExpr $I4 = $actpagedup

#then run clusteranalysis
MxClusterDetection $actpagedup $maskdup 0.01 1 $outname cluster probability resultgroup

MxDeletePage {$maskdup $actpagedup} 
MxSetCurrentPage $refpagedup
MxLoadLUT $refpagedup "/tmp/${USER}templut"

if {$annotate==1} {

    set channel [open $outname "r"]
    set found 0
    set noexit 1 
    MxGetCurrentImage image

    while {$noexit} {

	gets $channel line
	set element [lindex $line 0]

	if  {$element=="Cluster#"} {
	    #found start of the business
	    while {$line!=""} {
		gets $channel line
		set clustno [lindex $line 0]
		set size    [lindex $line 1]
		set x	    [lindex $line 3]
		set y 	    [lindex $line 4]
		set z	    [lindex $line 5]
		if {$size>2} {
		    if {$clustno<10} {
		    MxDrawTextGraphic $image $clustno $x $y $z 12 8 text
		    } else {
		    MxDrawTextGraphic $image $clustno $x $y $z 16 8 text
		    }
		} 
	    }
	    set noexit 0
	}
	
    }
 }
#end annotate if


#OK then place 3D Marker 

while (1) {

MxSetGraphicToolMode 3DMarker
set reply [YesNoWidget "Move to cluster location click Report" Report Exit]
if {$reply == 0} {
 break }

MxGetCurrentGraphic marker
MxGet3DMarkerProperties $marker Properties
set mx [keylget Properties X]
set my [keylget Properties Y]
set mz [keylget Properties Z]

MxGetVoxel $cluster $mx $my $mz value
ScriptUpdate "The cluster number is : $value"
MxDeleteGraphic $marker

}

