#

#   iso - this changes the voxel size to make the image isotropic
#
#   Stephen Smith, FMRIB Image Analysis Group
#
#   Copyright (C) 1999-2000 University of Oxford
#
#   TCLCOPYRIGHT

if { [ info exists env(FSLDIR) ] } {
    set FSLDIR $env(FSLDIR)
} else {
    set FSLDIR $PXHOME/fsl
}

source ${FSLDIR}/tcl/medxstart.tcl
set auto_path [ linsert $auto_path 0 [ file dirname [ info script ] ] ]

proc makeiso { Folder Page zadd } {

    #{{{ find image properties

    set depth 1
    MxGetDimensions $Page width height depth

    puts "Dimensions=($width,$height,$depth)"

    set zscale 1
    MxGetScale $Page xscale yscale zscale

    puts "Voxel=($xscale,$yscale,$zscale)"

#}}}
    #{{{ setup scaling

    if { $xscale < $yscale } {
	set cubel $xscale
    } else {
	set cubel $yscale
    }	
    if { $zscale < $cubel } {
	set cubel $zscale
    }

    puts "Cubel side length=$cubel"

    set xscaling [ expr double($xscale) / $cubel ]
    set yscaling [ expr double($yscale) / $cubel ]
    set zscaling [ expr double($zscale) / $cubel ]

    puts "Scaling=($xscaling,$yscaling,$zscaling)"

#}}}
    #{{{ COMMENT setup new dimensions OLD

#    set newwidth  [ expr int( $width  * $xscaling ) ]
#    set newheight [ expr int( $height * $yscaling ) ]
#    set newdepth  [ expr int( $depth  * $zscaling ) ]
#
#    puts "New Dimensions=($newwidth,$newheight,$newdepth)"
#    
#    if { $zadd > -0.001 } {
#	set zadd [ expr int( double($zadd) / $cubel ) ]
#	puts "Adding $zadd slices to top and bottom:"
#
#	set newdepth [ expr $newdepth + ( $zadd * 2 ) ]
#    } else {
#	set newdepth [ expr 0 - $zadd ]
#    }
#
#    puts "New Dimensions=($newwidth,$newheight,$newdepth)"
#

#}}}
    #{{{ setup new dimensions

    set newwidth  [ expr int( $width  * $xscaling ) ]
    set newheight [ expr int( $height * $yscaling ) ]
    set newdepth  [ expr int( $depth  * $zscaling ) ]

    puts "New Dimensions=($newwidth,$newheight,$newdepth)"
    
# this line took in zadd as mm rather than voxls - commented it out
#    set zadd [ expr int( double($zadd) / $cubel ) ]
    puts "Adding $zadd slices to top and bottom:"

    set newdepth [ expr $newdepth + ( $zadd * 2 ) ]

    puts "New Dimensions=($newwidth,$newheight,$newdepth)"

#}}}

    MxInterpolationMode BTL

    MxReslice $Page "{Cx [ expr ( $width - 1.0 ) / 2.0 ] } {Cy [ expr ( $height - 1.0 ) / 2.0 ] } \
	    {Cz [ expr ( $depth - 1.0 ) / 2.0 ] } {Rx 0} {Ry 0} {Rz 0} {Sx $xscaling} {Sy $yscaling} {Sz $zscaling} \
	    {Tx 0} {Ty 0} {Tz 0} {Dx $newwidth} {Dy $newheight} {Dz $newdepth } {RotMethod M}" IsoPage

    return $IsoPage
}
